=== modified file 'NEWS'
--- NEWS	2008-12-01 18:40:08 +0000
+++ NEWS	2008-12-01 23:20:22 +0000
@@ -23,6 +23,11 @@
     * Give proper error message for diff with non-existent dotted revno.
       (Marius Kruger, #301969)
 
+    * ``Transport.readv()`` defaults to not reading more than 100MB in a
+      single array. Further ``RemoteTransport.readv`` sets this to 5MB to
+      work better with how it splits its requests.
+      (John Arbash Meinel, #303538)
+
   DOCUMENTATION:
 
   API CHANGES:

=== modified file 'bzrlib/tests/test_transport.py'
--- bzrlib/tests/test_transport.py	2008-09-10 19:14:43 +0000
+++ bzrlib/tests/test_transport.py	2008-12-02 01:22:00 +0000
@@ -225,6 +225,16 @@
                    [(10, 10), (20, 10), (30, 50), (80, 100)],
                   )
 
+    def test_coalesce_default_limit(self):
+        # By default we use a 100MB max size.
+        ten_mb = 10*1024*1024
+        self.check([(0, 10*ten_mb, [(i*ten_mb, ten_mb) for i in range(10)]),
+                    (10*ten_mb, ten_mb, [(0, ten_mb)])],
+                   [(i*ten_mb, ten_mb) for i in range(11)])
+        self.check([(0, 11*ten_mb, [(i*ten_mb, ten_mb) for i in range(11)]),],
+                   [(i*ten_mb, ten_mb) for i in range(11)],
+                   max_size=1*1024*1024*1024)
+
 
 class TestMemoryTransport(TestCase):
 

=== modified file 'bzrlib/transport/__init__.py'
--- bzrlib/transport/__init__.py	2008-11-07 18:10:32 +0000
+++ bzrlib/transport/__init__.py	2008-12-01 23:20:22 +0000
@@ -797,6 +797,10 @@
         cur = _CoalescedOffset(None, None, [])
         coalesced_offsets = []
 
+        if max_size <= 0:
+            # 'unlimited', but we actually take this to mean 100MB buffer limit
+            max_size = 100*1024*1024
+
         for start, size in offsets:
             end = start + size
             if (last_end is not None

=== modified file 'bzrlib/transport/remote.py'
--- bzrlib/transport/remote.py	2008-11-21 00:23:54 +0000
+++ bzrlib/transport/remote.py	2008-12-01 23:20:22 +0000
@@ -323,7 +323,8 @@
         sorted_offsets = sorted(offsets)
         coalesced = list(self._coalesce_offsets(sorted_offsets,
                                limit=self._max_readv_combine,
-                               fudge_factor=self._bytes_to_read_before_seek))
+                               fudge_factor=self._bytes_to_read_before_seek,
+                               max_size=self._max_readv_bytes))
 
         # now that we've coallesced things, avoid making enormous requests
         requests = []

